#include "List.h"

template <typename TYPE>
List<TYPE>::iterator::iterator(Node *node)
: mNode(node) 
{
    mParent = mNode->getParent();
}

template <typename TYPE>
List<TYPE>::iterator::iterator()
: mNode(NULL)
{
}

template <typename TYPE>
List<TYPE>::iterator::~iterator()
{
}

template <typename TYPE>
const bool List<TYPE>::iterator::invalid() const
{
    return mNode == NULL;
}

template <typename TYPE>
const typename List<TYPE>::iterator
List<TYPE>::iterator::end() const
{
    if (invalid()) {
        return *this;
    } else {
        List *parent = mNode->getParent();
        Node *tail = parent->getTail();
        return (typename List<TYPE>::iterator)(tail);
    }
}

template <typename TYPE>
typename List<TYPE>::Node*
List<TYPE>::iterator::getNode()
{
    return mNode;
}

template <typename TYPE>
const typename List<TYPE>::Node*
List<TYPE>::iterator::getNode() const
{
    return mNode;
}

template <typename TYPE>
void List<TYPE>::iterator::setNode(Node *node)
{
    mNode = node;
    mParent = node->getParent();
}

template <typename TYPE>
List<TYPE> * List<TYPE>::iterator::getParent()
{
    return mParent;
}

template <typename TYPE>
const List<TYPE> * List<TYPE>::iterator::getParent() const
{
    return mParent;
}

template <typename TYPE>
TYPE * List<TYPE>::iterator::getElement()
{
    return (invalid()) ? NULL : mNode->getElem();
}

template <typename TYPE>
const TYPE * List<TYPE>::iterator::getElement() const
{
    return (invalid()) ? NULL : mNode->getElem();
}

template <typename TYPE>
typename List<TYPE>::iterator&
List<TYPE>::iterator::operator -- ()
{
    if (!invalid() && mNode != mParent->getHead()) {
        *this = List<TYPE>::iterator(mNode->getPrev());
    }
    return *this;
}

template <typename TYPE>
typename List<TYPE>::iterator
List<TYPE>::iterator::operator -- (int)
{
    if (!invalid() && mNode != mParent->getHead()) {
        *this = List<TYPE>::iterator(mNode->getPrev());
    }
    return *this;
}

template <typename TYPE>
typename List<TYPE>::iterator&
List<TYPE>::iterator::operator ++ ()
{
    if (!invalid() && mNode != mParent->getTail()) {
        *this = List<TYPE>::iterator(this->getNext());
    }
    return *this;
}

template <typename TYPE>
typename List<TYPE>::iterator
List<TYPE>::iterator::operator ++ (int)
{
    if (!invalid() && mNode != mParent->getTail()) {
        *this = typename List<TYPE>::iterator(mNode->getNext());
    }
    return *this;
}

template <typename TYPE>
typename List<TYPE>::iterator&
List<TYPE>::iterator::operator = (const typename List<TYPE>::iterator& that)
{
    mParent = that.mParent;
    mNode = that.mNode;
    return *this;
}

template <typename TYPE>
const bool
List<TYPE>::iterator::
operator == (const typename List<TYPE>::iterator& that) const
{
    return mNode == that.mNode;
}

template <typename TYPE>
const bool
List<TYPE>::iterator::
operator != (const typename List<TYPE>::iterator& that) const
{
    return mNode != that.mNode;
}
