#include "List.h"

#include "Animal.h"
#include "Cow.h"

#include <string>
#include <iostream>

#include <assert.h>

using namespace std;

const int MaxSize = 10;

typedef List<Animal> Farm;
typedef struct CallbackData
{
    Farm *farm;
} CallbackData;

typedef void (*CallbackFn)(void *);

void init(void *data)
{
    CallbackData *cbData = reinterpret_cast<CallbackData*>(data);
    cbData->farm = new Farm();
}

void cleanup(void *data)
{
    CallbackData *cbData = reinterpret_cast<CallbackData*>(data);
    Farm *farm = cbData->farm;
    Farm::iterator it = farm->getIterator();
    for (; it != it.end(); it++) {
        Animal *animal = it.getElement();
        delete animal;
    }
    delete farm;
}

void run(CallbackFn *testCase,
         const char ** testNames,
         const int& testCaseSize,
         void *testCaseData)
{
    for (int i = 0; i < testCaseSize; i++) {
        cout << "Running " << testNames[i] << "..." << endl;
        init(testCaseData);
        testCase[i](testCaseData);
        cleanup(testCaseData);
        cout << testNames[i] << " succeeded!" << endl;
    }
}

// Test Case 1:  Add animals to farm.
void testAdd(void *data)
{
    CallbackData *cbData = reinterpret_cast<CallbackData*>(data);
    Farm *farm = cbData->farm;

    farm->append(new Cow("Hilbert"));
    farm->append(new Cow("Alice"));
    farm->append(new Cow("Madeye"));

    // Make sure the farm size is three:
    const int size = farm->getSize();
    cout << "Farm size after add: " << farm->getSize() << endl;
    assert(size == 3);

    // Check to make sure that our cow identities are in tact:
    Farm::iterator it = farm->getIterator();
    Animal *animal = it.getElement();
    assert(animal->getType() == "Cow");
    assert(animal->getName() == "Hilbert");
    it++;

    animal = it.getElement();
    assert(animal->getType() == "Cow");
    assert(animal->getName() == "Alice");
    it++;

    animal = it.getElement();
    assert(animal->getType() == "Cow");
    assert(animal->getName() == "Madeye");
    it++;

    assert(it == it.end());
    assert(Cow::getTotalCount() == 3);
}

// Test Case 2:  Remove animals from farm.
void testRemove(void *data)
{
    CallbackData *cbData = reinterpret_cast<CallbackData*>(data);
    Farm *farm = cbData->farm;

    for (int i = 0; i < MaxSize; i++)
    {
        farm->append(new Cow());
    }

    const int size = farm->getSize();
    assert(size == MaxSize);

    int idx = 0;
    Farm::iterator it = farm->getIterator();
    while(it != it.end()) {
        Animal *animal = it.getElement();
        farm->remove(it);
        delete animal;
    }

    assert(Cow::getTotalCount() == 0);
    assert(farm->getSize() == 0);
}

int main(int argc, char **argv)
{
    CallbackData data;

    // TODO: Try defining a TestCase class:
    const char * testNames [] = {"testAdd", "testRemove"};
    CallbackFn testCases [] = {testAdd , testRemove};
    run(testCases, testNames, 2, &data);

    cout << "Test cases ran successfully.  Exiting." << endl;
    return 0;
}
